#include "time.h"
#include "rtc.h"
#include "util.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

static const char monthDays[]={31,28,31,30,31,30,31,31,30,31,30,31};

time_t	now;

void update_time(void)
{
	struct tm t;
	rtc_read_t(&t);
	// make time_t
	CheckTime(&t);
	now = mktime(&t);
}

time_t time(time_t *t)
{
	update_time();
	if (t)
		*t = now;
	return now;
}

const char * MonthNames[]={
	"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
	"JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

static char ascTimeBuffer[32];

void CheckTime(struct tm *t)
{
    if (t->tm_sec>59) t->tm_sec=59;
    if (t->tm_min>59) t->tm_min=59;
    if (t->tm_hour>23) t->tm_hour=23;
    if (t->tm_wday>6) t->tm_wday=6;
    if (t->tm_mday<1) t->tm_mday=1;
    else if (t->tm_mday>31) t->tm_mday=31;
    if (t->tm_mon>11) t->tm_mon=11;
    if (t->tm_year<0) t->tm_year=0;
}

// format the time into "2001-FEB-17 17:45:23"
char *asctime(struct tm *t)
{
	CheckTime(t);
	sprintf (ascTimeBuffer, "%04d-%s-%02d %02d:%02d:%02d",
		t->tm_year+1900,
		MonthNames[t->tm_mon],
		t->tm_mday,
		t->tm_hour, t->tm_min, t->tm_sec
		);
	return ascTimeBuffer;
}

// format the date into "2001-FEB-17"
char *datestr(struct tm *t)
{
	CheckTime(t);
	sprintf (ascTimeBuffer, "%04d-%s-%02d %02d:%02d:%02d",
		t->tm_year+1900,
		MonthNames[t->tm_mon],
		t->tm_mday,
		t->tm_hour, t->tm_min, t->tm_sec
		);
	ascTimeBuffer[11] = 0;
	return ascTimeBuffer;
}

// format the time into "17:45:23"
char *timestr(struct tm *t)
{
	CheckTime(t);
	sprintf (ascTimeBuffer, "%04d-%s-%02d %02d:%02d:%02d",
		t->tm_year+1900,
		MonthNames[t->tm_mon],
		t->tm_mday,
		t->tm_hour, t->tm_min, t->tm_sec
		);
	return &ascTimeBuffer[12];
}



#define LEAP_YEAR(year) ((year%4)==0)

struct tm * gmtime(time_t *t)
{
	static struct tm lastTime;
	unsigned long epoch = *t;
	unsigned int year;
	unsigned char month, monthLength;
	unsigned long days;

	lastTime.tm_sec = epoch % 60;
	epoch /= 60;
	lastTime.tm_min = epoch % 60;
	epoch /= 60;
	lastTime.tm_hour = epoch % 24;
	epoch /= 24;
	lastTime.tm_wday = (epoch + 4) % 7;

	year = 1970;
	days = 0;

	while((days += (LEAP_YEAR(year) ? 366 : 365)) <= epoch)
		year++;

	lastTime.tm_year = year - 1900;

	days -= LEAP_YEAR(year) ? 366 : 365;
	epoch -= days;
	lastTime.tm_yday = epoch;

	for (month = 0; month < 12; month++)
	{
		if (month == 1) // feb
			if (LEAP_YEAR(year))
				monthLength = 29;
			else
				monthLength = 28;
		else
			monthLength = monthDays[month];

		if (epoch >= monthLength)
			epoch -= monthLength;
		else
		{
			lastTime.tm_mon = month;
			lastTime.tm_mday = epoch+1;
			break;
		}
	}
	return &lastTime;
}

time_t mktime(struct tm *t)
{
	int year, month, i;
	unsigned long seconds;

	year    = t->tm_year + 1900;
	month   = t->tm_mon;
	seconds = (unsigned long)(year - 1970) * (60*60*24L*365);

	for (i = 1970; i < year; i++)
		if (LEAP_YEAR(i))
			seconds += (unsigned long)60*60*24L;

	// add days for this year
	for (i = 0; i < month; i++)
		if (i == 1 && LEAP_YEAR(year))
			seconds += (unsigned long)60*60*24L*29;
		else
			seconds += (unsigned long)60*60*24L*monthDays[i];

	seconds += (unsigned long)(t->tm_mday-1) * 60*60*24L;
	seconds += (unsigned long)t->tm_hour * 60*60L;
	seconds += (unsigned long)t->tm_min * 60L;
	seconds += (unsigned long)t->tm_sec;
	return seconds;
}


